package cn.rankin.apiweb.service.collection;

import cn.rankin.apiweb.code.ApiWebCode;
import cn.rankin.apiweb.service.product.ProductClient;
import cn.rankin.apiweb.service.product.ProductService;
import cn.rankin.apiweb.service.user.UserService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.model.BaseCode;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.data.api.app.dto.ItemSearchDTO;
import cn.rankin.data.api.app.vo.ItemVo;
import cn.rankin.data.api.product.entity.MerchantProduct;
import cn.rankin.data.api.user.dto.CollectionDTO;
import cn.rankin.data.api.user.entity.Collection;
import cn.rankin.data.api.user.entity.UserTagProductRelation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;


@Service
@Slf4j
public class CollectionService {

    protected static final int MAX_SIZE = 1000;

    @Autowired
    private CollectionClient collectionClient;

    @Autowired
    private ProductClient productClient;

    @Autowired
    private ProductService productService;

    @Autowired
    private UserService userService;

    public Boolean isCollected(String userId, String productId) {
        APIResult<Collection> apiResult = collectionClient.getCollection(userId, productId);
        if (apiResult.getSuccess() && apiResult.getData() != null) {
            return Boolean.TRUE;
        }
        return Boolean.FALSE;
    }

    public APIResult<Boolean> put(String userId, String merchantId, String productId) {
        APIResult<MerchantProduct> apiResult = productService.getMerchantProduct(productId, merchantId);
        if (!apiResult.getSuccess()) {
            return APIResult.error(new BaseCode(apiResult.getCode(), apiResult.getMessage()));
        }

        MerchantProduct merchantProduct = apiResult.getData();
        if (merchantProduct == null) {
            return APIResult.error(ApiWebCode.NOT_EXISTS);
        }

        CollectionDTO collectionDTO = new CollectionDTO();
        collectionDTO.setPid(productId);
        collectionDTO.setUserId(userId);
        collectionDTO.setType(merchantProduct.getType());
        collectionDTO.setStatus(BaseStatusEnum.NORMAL);

        return collectionClient.put(collectionDTO);
    }

    public APIResult<Boolean> delete(String userId, String productId) {
        APIResult<Boolean> apiResult = collectionClient.delete(userId, productId);
        return apiResult;
    }

    public APIResult<Page<ItemVo>> getUserCollection(String userId, String merchantId, Integer pageNo, Integer pageSize) {
        APIResult<Page<Collection>> collectionApiResult = collectionClient.getPage(userId, pageNo, pageSize);
        if (!collectionApiResult.getSuccess()) {
            return APIResult.error(new BaseCode(collectionApiResult.getCode(), collectionApiResult.getMessage()));
        }
        Page<Collection> collectionPage = collectionApiResult.getData();
        List<Collection> collectionList = collectionPage.getList();
        if (CollectionUtils.isEmpty(collectionList)) {
            return APIResult.error(new BaseCode(collectionApiResult.getCode(), collectionApiResult.getMessage()));
        }

        Page<ItemVo> page = new Page(collectionPage.getTotalSize(), collectionPage.getPageNo(), collectionPage.getPageSize());

        List<String> productIdList = new ArrayList<>();
        collectionList.forEach(collection -> productIdList.add(collection.getPid()));
        APIResult<List<ItemVo>> itemApiResult = productService.findItemByIds(merchantId, productIdList);
        if (!itemApiResult.getSuccess()) {
            return APIResult.error(new BaseCode(itemApiResult.getCode(), itemApiResult.getMessage()));
        }
        List<ItemVo> itemVoList = itemApiResult.getData();
        page.setList(itemVoList);

        return APIResult.ok(page);
    }

    public APIResult<Page<ItemVo>> getUserCollection(String userId, String merchantId, String tagId, Integer pageNo, Integer pageSize) {
        APIResult<List<Collection>> collectionApiResult = collectionClient.getUserCollections(userId, MAX_SIZE);
        if (!collectionApiResult.getSuccess()) {
            return APIResult.error(new BaseCode(collectionApiResult.getCode(), collectionApiResult.getMessage()));
        }
        List<Collection> collectionList = collectionApiResult.getData();
        if (CollectionUtils.isEmpty(collectionList)) {
            return APIResult.ok(new ArrayList<>());
        }
        List<String> productIdList = new ArrayList<>();
        for (Collection collection : collectionList) {
            String productId = collection.getPid();
            productIdList.add(productId);
        }

        ItemSearchDTO itemSearchDTO = new ItemSearchDTO();
        itemSearchDTO.setMerchantId(merchantId);
        itemSearchDTO.setTagId(tagId);
        itemSearchDTO.setProductIdList(productIdList);
        itemSearchDTO.setPageNo(pageNo);
        itemSearchDTO.setPageSize(pageSize);

        return productService.findItemPageByIds(itemSearchDTO);
    }

    public APIResult<Page<ItemVo>> getUserConllectionByUserTag(String userId, String merchantId, String userTagId,
                                                               Integer pageNo, Integer pageSize) {
        APIResult<List<Collection>> collectionApiResult = collectionClient.getUserCollections(userId, MAX_SIZE);
        if (!collectionApiResult.getSuccess()) {
            return APIResult.error(new BaseCode(collectionApiResult.getCode(), collectionApiResult.getMessage()));
        }
        List<Collection> collectionList = collectionApiResult.getData();
        if (CollectionUtils.isEmpty(collectionList)) {
            return APIResult.ok(new ArrayList<>());
        }
        List<String> productIdList = new ArrayList<>();
        for (Collection collection : collectionList) {
            String productId = collection.getPid();
            productIdList.add(productId);
        }

        Page<UserTagProductRelation> resultPage = userService.findRelations(userTagId, productIdList, pageNo, pageSize);

        if(resultPage == null){
            return APIResult.ok(new ArrayList());
        }

        List<ItemVo> itemVos = new ArrayList<>();
        List<UserTagProductRelation> userTagProductRelations = resultPage.getList();
        userTagProductRelations.forEach(userTagProductRelation -> {
            String pid = userTagProductRelation.getPid();
            ItemVo itemVo = productClient.findItemByPid(merchantId, pid);
            if(itemVo == null){
                log.info("not found item by pid , pid={}",pid);
            }else{
                itemVos.add(itemVo);
            }
        });

        Page<ItemVo> page = new Page<>();
        page.setPageSize(pageSize);
        page.setPageNo(pageNo);
        page.setTotalSize((int)resultPage.getTotalSize());
        page.setList(itemVos);

        return APIResult.ok(page);
    }
}
