package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.DateUtil;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.dto.TagDTO;
import cn.rankin.data.api.product.entity.*;
import cn.rankin.productservice.code.ProductServiceAPICode;
import cn.rankin.productservice.repository.MerchantProductRepository;
import cn.rankin.productservice.repository.TagGroupRepository;
import cn.rankin.productservice.repository.TagRepository;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Slf4j
@Service
public class TagService {

    @Autowired
    private TagRepository tagRepository;

    @Autowired
    private TagGroupRepository tagGroupRepository;

    @Autowired
    private MerchantProductRepository merchantProductRepository;

    @Autowired
    private MerchantProductTagRelationService merchantProductTagRelationService;

    @Autowired
    private GoodsService goodsService;

    public List<Tag> findByGroupId(String groupId) {
        List<Tag> tagList = tagRepository.findByGroupId(groupId, BaseStatusEnum.NORMAL);
        setGroupInfo(tagList);
        return tagList;
    }

    /*add    2018-05-17 */
    public List<Tag> findByTypeCode(String typeCode,String merchantId) {
        List<Tag> tagList = tagRepository.findByTypeCode(typeCode,merchantId, BaseStatusEnum.NORMAL);
        return tagList;
    }

    public List<Tag> findByTypeCode(String typeCode) {
        List<Tag> tagList = tagRepository.findByTypeCode(typeCode);
        return tagList;
    }


    public void setGroupInfo(List<Tag> tagList) {
        if (CollectionUtils.isEmpty(tagList)) return;
        List<String> groupIdList = new ArrayList<>();
        tagList.forEach(tag -> {
            String groupId = tag.getGroupId();
            if (!groupIdList.contains(groupId)) {
                groupIdList.add(groupId);
            }
        });

        List<TagGroup> groupList = tagGroupRepository.findByIds(groupIdList);
        Map<String, TagGroup> tagGroupMap = ListUtil.convert(groupList, "id", TagGroup.class);

        for (Tag tag : tagList) {
            String groupId = tag.getGroupId();
            TagGroup tagGroup = tagGroupMap.get(groupId);
            if (tagGroup != null) {
                tag.setGroupName(tagGroup.getName());
            }
        }
    }

    public List<MerchantProduct> getMerchantProductList(String tagId, String merchantId) {
        List<MerchantProductTagRelation> relationList = merchantProductTagRelationService.findByTagId(tagId, BaseStatusEnum.NORMAL);
        if (CollectionUtils.isEmpty(relationList)) {
            return new ArrayList<>();
        }

        List<String> productIdList = new ArrayList<>();
        relationList.forEach(relation -> productIdList.add(relation.getPid()));
        List<MerchantProduct> merchantProductList = merchantProductRepository.findByPidsAndMerchantId(productIdList, merchantId);
        Map<String, MerchantProduct> merchantProductMap = ListUtil.convert(merchantProductList, "pid", MerchantProduct.class);

        List<MerchantProduct> sortMerchantProductList = new ArrayList<>();
        for (String productId : productIdList) {
            MerchantProduct merchantProduct = merchantProductMap.get(productId);
            if (merchantProduct != null) {
                sortMerchantProductList.add(merchantProduct);
            }
        }
        return sortMerchantProductList;
    }

    // 搜索标签
    public APIResult<Page<Tag>> search(Tag tag, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = tagRepository.count(tag);
        Page<Tag> page = new Page(count, pageNo, pageSize);
        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Tag> tagList = tagRepository.find(tag, page.getStart(), page.getPageSize(), JpaSortUtil.sort(sort));
        setGroupInfo(tagList);
        page.setList(tagList);

        return APIResult.ok(page);
    }

    // 标签详情
    public APIResult<Tag> getTag(String tagId) {
        Tag tag = tagRepository.find(tagId);
        String merchantId = tag.getMerchantId();

        List<MerchantProduct> merchantProductList = getMerchantProductList(tagId, merchantId);
        tag.setProductList(merchantProductList);
        return APIResult.ok(tag);
    }

    @Transactional
    public APIResult<Tag> create(TagDTO tagDTO) {
        Tag tag = convert(tagDTO);
        //新版本
        String merchantId = tagDTO.getMerchantId();
        //弃用字段，此处为处理数据库不能为空
        tag.setGroupId("groupId"+DateUtil.format(new Date(),"yyyyMMddHHmmss"));
        tag.setMerchantId(merchantId);
        Tag result = tagRepository.save(tag);

        String tagId = result.getId();
        List<String> productIdList = tagDTO.getProductList();
        merchantProductTagRelationService.updateByTagId(tagId, merchantId, productIdList);

        List<MerchantProduct> merchantProductList = getMerchantProductList(tagId, merchantId);
        tag.setProductList(merchantProductList);

        return APIResult.ok(tag);
    }

    @Transactional
    public APIResult<Tag> update(TagDTO tagDTO) {
        Tag tag = convert(tagDTO);
        Tag result = tagRepository.update(tag);

        String tagId = result.getId();
        String merchantId = result.getMerchantId();
        List<String> productIdList = tagDTO.getProductList();
        merchantProductTagRelationService.updateByTagId(tagId, merchantId, productIdList);

        List<MerchantProduct> merchantProductList = getMerchantProductList(tagId, merchantId);
        tag.setProductList(merchantProductList);

        return APIResult.ok(tag);
    }

    @Transactional
    public APIResult<Boolean> delete(String tagId) {
        Integer count = tagRepository.deleteById(tagId);
        if (count > 0) {
            return APIResult.ok(true);
        }
        return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
    }

    @Transactional
    public void sortTag(String groupId, List<String> tagIdList) {
        List<Tag> tagList = tagRepository.findByGroupId(groupId, BaseStatusEnum.NORMAL);
        sortTag(tagIdList, tagList);
        tagRepository.update(tagList);
    }



    @Transactional
    public void sortTagByTypeCode(String typeCode, List<String> tagIdList) {
        List<Tag> tagList = tagRepository.findByTypeCode(typeCode);
        sortTag(tagIdList, tagList);
        tagRepository.update(tagList);
    }

    private void sortTag(List<String> tagIdList, List<Tag> tagList) {
        for (Tag tag : tagList) {
            String tagId = tag.getId();
            if (!tagIdList.contains(tagId)) {
                tag.setStatus(BaseStatusEnum.DEL);
                continue;
            }
            Integer index = tagIdList.indexOf(tagId);
            tag.setSort(index);
        }
    }

    public Tag getTagInfo(String tagId) {
        Tag tag = tagRepository.find(tagId);
        String merchantId = tag.getMerchantId();

        List<MerchantProduct> merchantProductList = getMerchantProductList(tagId, merchantId);
        tag.setProductList(merchantProductList);
        return tag;
    }

    public Tag find(String tagId) {
        Tag tag = tagRepository.find(tagId);
        return tag;
    }
}
